"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.stringify = exports.parse = void 0;
// perhaps in the future we can use @bevry/json's toJSON and parseJSON and JSON.stringify to support more advanced types
/** Parse an envfile string. */
function parse(src) {
    const result = {};
    const lines = splitInLines(src);
    for (const line of lines) {
        const match = line.match(/^([^=:#]+?)[=:]((.|\n)*)/);
        if (match) {
            const key = match[1].trim();
            const value = match[2].trim().replace(/['"]+/g, '');
            result[key] = value;
        }
    }
    return result;
}
exports.parse = parse;
/** Turn an object into an envfile string. */
function stringify(obj) {
    let result = '';
    for (const [key, value] of Object.entries(obj)) {
        if (key) {
            const line = `${key}=${jsonValueToEnv(value)}`;
            result += line + '\n';
        }
    }
    return result;
}
exports.stringify = stringify;
function splitInLines(src) {
    return src
        .replace(/("[\s\S]*?")/g, (_m, cg) => {
        return cg.replace(/\n/g, '%%LINE-BREAK%%');
    })
        .split('\n')
        .filter((i) => Boolean(i.trim()))
        .map((i) => i.replace(/%%LINE-BREAK%%/g, '\n'));
}
function jsonValueToEnv(value) {
    let processedValue = String(value);
    processedValue = processedValue.replace(/\n/g, '\\n');
    processedValue = processedValue.includes('\\n')
        ? `"${processedValue}"`
        : processedValue;
    return processedValue;
}
